import Qt 4.7

Rectangle {
    id: rootRect
    width: 200
    height: 200

    Rectangle {
        id: greenRect
        width: 92.5
        height: 92.5
        color: "green"
        anchors.left: parent.left
        anchors.top: parent.top
        anchors.margins: 5
        Text {
            text: parent.width + "," + parent.height
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.verticalCenter: parent.verticalCenter
        }
    }

    Rectangle {
        id: redRect
        height: 92.5
        color: "red"
        anchors.right: parent.right
        anchors.left: greenRect.right
        anchors.top: parent.top
        anchors.margins: 5
        Text {
            text: parent.width + "," + parent.height
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.verticalCenter: parent.verticalCenter
        }
    }

    Rectangle {
        id: yellowRect
        color: "yellow"
        anchors.top: greenRect.bottom
        anchors.bottom: parent.bottom
        anchors.right: parent.right
        anchors.left: parent.left
        anchors.margins: 5
        Text {
            text: parent.width + "," + parent.height
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.verticalCenter: parent.verticalCenter
        }
    }

    states: State {
        name: "swapped"
        when: mouseArea.pressed

        // When anchors are changed, the previous anchors need to
        // be cleared by setting them to "undefined". Otherwise
        // blueRect would still be anchored to parent bottom-right
        // and thus would fill the whole parent
        AnchorChanges {
            target: yellowRect
            anchors.left: parent.left
            anchors.top: parent.top
            anchors.bottom: undefined
            anchors.right: undefined
        }

        // Yellow rectangle is moved to top-left corner and it's
        // bottom and right anchors are removed. Since it's no longer
        // fully anchored, it needs a width and height in the new state.
        PropertyChanges {
            target: yellowRect
            width: 120
            height: 120
        }

        // Green rectangle is moved to top-right corner and re-anchored
        // to yellow rectangle that is moved to top-left corner.
        // Previous bottom anchor (parent.bottom) is removed
        AnchorChanges {
            target: greenRect
            anchors.right: parent.right
            anchors.left: yellowRect.right
            anchors.top: parent.top
            anchors.bottom: undefined
        }

        // Since the bottom anchor was removed, the green rectangle needs height
        // Width also needs to be adjusted. Width is implicitly changed due to
        // the anchor changes. When switching back, the width doesn't change back
        // unless we explicitly specify a width change during state changes
        // @TODO: Check if that is a QML bug
        PropertyChanges {
            target: greenRect
            width: 12345 // Demonstrate by commenting this out
            height: 120
        }

        // Red rectangle is fully anchored in the new state.
        AnchorChanges {
            target: redRect
            anchors.top: yellowRect.bottom
            anchors.right: parent.right
            anchors.bottom: parent.bottom
            anchors.left: parent.left
        }

        // Red rectangle height suffers from the same problem as green above
        PropertyChanges {
            target: redRect
            height: 12345
        }
    }

    // AnchorAnimation animates all changes by default
    // targets property can be used to apply the animation
    // to certain subset of elements. In this case yellow
    // is a bit slower
    transitions: Transition {
        AnchorAnimation {
            targets: [ redRect, greenRect ]
        }
        AnchorAnimation {
            targets: [ yellowRect ]
            duration: 500
        }
    }

    MouseArea {
        anchors.fill: parent
        id: mouseArea
    }

}
