import Qt 4.7

Rectangle {
    width: 300
    height: 400

    // Top part, contains a text input and Submit button.
    // Width follows parent, height is fixed
    // filterArea.message is updated when Submit-button is pressed
    FilterArea {
        id: filterArea
        anchors.top: parent.top
        anchors.left: parent.left
        anchors.right: parent.right
        height: 40
    }

    // Area that slides in from the left side when clicked
    SlideArea {
        id: slideArea
        width: colorArea.x - 5
        anchors.top: filterArea.bottom
        anchors.bottom: statusArea.top
    }

    // List of items slides away to the right when SlideArea
    // slides in.
    // SlideArea changes it's relativePosition from 0 to 1
    // during the slide and opacity is bound to that
    ListArea {
        id: listArea
        anchors.top: filterArea.bottom
        anchors.bottom: statusArea.top
        anchors.left: slideArea.right
        anchors.right: colorArea.left
        opacity: 1 - slideArea.relativePosition
    }

    // ColorArea could also be sub-element of ListArea
    // as the exercise didn't specify how it'll be used
    // colorArea.selectedColor will be updated when user
    // clicks on one of the colored rectangles.
    ColorArea {
        id: colorArea
        anchors.top: filterArea.bottom
        anchors.bottom: statusArea.top
        anchors.right: parent.right
        width: childrenRect.width
        onColorSelected: listArea.colorSelected(selectedColor)
    }

    // Status area text is bound to filterArea.message
    // A better solution would be to change it via script,
    // since now the ColorArea updates are routed via
    // filterArea.message
    StatusArea {
        id: statusArea
        anchors.bottom: parent.bottom
        anchors.left: parent.left
        anchors.right: parent.right
        height: 25
        text: filterArea.message
    }

}
