import Qt 4.7

Rectangle {
    id: rootRect
    anchors.margins: 5
    border.width: 1
    radius: 5

    // Component.onCompleted property can be attached to any QML object
    // It is run when component is loaded.
    Component.onCompleted: {
        // This adds random names into model.
        var firstNames = [ "Oliver", "Jack", "Harry", "Charlie", "Alfie", "Thomas", "Joshua", "William", "Daniel", "James" ]
        var lastNames = [ "Smith", "Jones", "Taylor", "Brown", "Williams", "Wilson", "Johnson", "Davis", "Robinson", "Wright" ]
        for (var i = 0; i < 50; i++) {
            var elem = {
                firstName: firstNames[Math.round(Math.random() * (firstNames.length - 1))],
                lastName: lastNames[Math.round(Math.random() * (lastNames.length - 1))],
                tagColor: ""
            };
            listModel.append(elem);
        }
    }

    // When a color is selected, the currently selected model element is updated
    function colorSelected(tagColor) {
        listModel.setProperty(view.currentIndex, "tagColor", tagColor);
    }

    // Model is initially empty and filled dynamically
    ListModel {
        id: listModel
    }

    // Note that view clip must be enabled in order to prevent
    // child elements to be drawn outside its borders.
    // View is also given focus, so it can be navigated with
    // up and down keys
    ListView {
        id: view
        clip: true
        anchors.fill: parent
        anchors.margins: 5
        model: listModel
        delegate: ListDelegate {}
        highlight: listHighlight
        focus: true
    }

    // Delegate component draws the elements of the list
    // Note that the delegate is drawn on front of the
    // highlight, so if delegate is fully opaque, highlight
    // cannot be seen
    // In this case the delegate is text, so has no background fill

    // List hightlight delegate draws a border and colored
    // background for the currently selected item
    Component {
        id: listHighlight
        Rectangle {
            // Border goes out of bounds if width is not adjusted
            width: view.width - 1
            color: "lightblue"
            radius: 3
            border.width: 1
        }
    }

}
